package coronafana

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"reflect"
	"strings"
	"time"

	"git.paulbsd.com/paulbsd/coronafana/src/config"
	"github.com/cretz/bine/tor"
	"xorm.io/xorm"
)

// GetData fetch data from open data portal
func GetData(cfg config.Config, d *tor.Dialer) (cr Coronafana, err error) {
	var client = &http.Client{Transport: &http.Transport{DialContext: d.DialContext}}

	log.Println("Getting JSON file ...")

	if cfg.CoronaURL == "" {
		err = fmt.Errorf("CoronaURL is not defined")
		return
	}

	resp, err := client.Get(cfg.CoronaURL)
	if err != nil {
		return
	}

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return
	}
	defer resp.Body.Close()

	err = json.Unmarshal(body, &cr)
	if err != nil {
		return
	}

	log.Println("JSON file fetched.")

	return
}

// InsertData insert data to MySQL / MariaDB
func InsertData(cfg config.Config, data interface{}) (err error) {
	var tx *xorm.Session
	var i int

	var cd []interface{}
	var parseddate time.Time

	tx = cfg.Db.NewSession()
	defer tx.Close()

	log.Println("Getting max dates")

	err = cfg.Db.Desc("date").Limit(1, cfg.Diffdays).Find(&data)

	if len(cd) > 0 {
		if reflect.TypeOf(data) == reflect.TypeOf(new(Coronaglobaldata)) {
			datac := data.([]Coronaglobaldata)
			parseddate = datac[0].Date
		} else if reflect.TypeOf(data) == reflect.TypeOf(new(Coronapaysdata)) {
			datac := data.([]Coronapaysdata)
			parseddate = datac[0].Date
		}
		if err != nil {
			return
		}
	}

	tx.Begin()
	log.Println("Start inserting data ...")

	if reflect.TypeOf(data) == reflect.TypeOf([]Coronaglobaldata{}) {
		for _, dt := range data.([]Coronaglobaldata) {
			if dt.Date, err = GetParsedDate(dt.DateString); dt.Date.After(parseddate) {
				_, err = tx.Insert(&dt)
				if err != nil {
					if !strings.Contains(err.Error(), "Error 1062") {
						return
					}
				}
				i++
			}
		}
	} else if reflect.TypeOf(data) == reflect.TypeOf([]Coronapaysdata{}) {
		for _, dt := range data.([]Coronapaysdata) {
			if dt.Date, err = GetParsedDate(dt.DateString); dt.Date.After(parseddate) {
				_, err = tx.Insert(&dt)
				if err != nil {
					if !strings.Contains(err.Error(), "Error 1062") {
						return
					}
				}
				i++
			}
		}
	}

	err = tx.Commit()
	if err != nil {
		return
	}

	if i > 0 {
		log.Println(fmt.Sprintf("%d entries inserted", i))
	} else {
		log.Println("No entry inserted")
	}

	return
}

// GetParsedDate returns parsed date as time.Time
func GetParsedDate(date string) (res time.Time, err error) {
	res, err = time.Parse("2006-01-02T15:04:05", date)
	return
}

// Coronafana is the main struct
type Coronafana struct {
	Source           string `json:"Source"`
	Information      string `json:"Information"`
	Utilisation      string `json:"Utilisation"`
	MaxDateGlobal    time.Time
	MaxDatePays      time.Time
	Coronaglobaldata []Coronaglobaldata `json:"GlobalData"`
	Coronapaysdata   []Coronapaysdata   `json:"PaysData"`
}

// Coronaglobaldata relates to global data
type Coronaglobaldata struct {
	ID            int       `xorm:"not null pk autoincr INT(8)"`
	Date          time.Time `xorm:"not null unique(date) DATETIME"`
	Infection     int       `json:"Infection" xorm:"default NULL INT(8)"`
	Deces         int       `json:"Deces" xorm:"default NULL INT(8)"`
	Guerisons     int       `json:"Guerisons" xorm:"default NULL INT(8)"`
	Tauxdeces     float64   `json:"TauxDeces"`
	Tauxguerison  float64   `json:"TauxGuerison"`
	Tauxinfection float64   `json:"TauxInfection"`
	DateString    string    `json:"Date" xorm:"-"`
}

// Coronapaysdata relates to country data
type Coronapaysdata struct {
	ID            int       `xorm:"not null pk autoincr INT(8)"`
	Date          time.Time `xorm:"not null unique(date) DATETIME"`
	Pays          string    `json:"Pays" xorm:"not null unique(date) VARCHAR(50)"`
	Infection     int       `json:"Infection" xorm:"default NULL INT(8)"`
	Deces         int       `json:"Deces" xorm:"default NULL INT(8)"`
	Guerisons     int       `json:"Guerisons" xorm:"default NULL INT(8)"`
	Tauxdeces     float64   `json:"TauxDeces" xorm:"default NULL FLOAT"`
	Tauxguerison  float64   `json:"TauxGuerison" xorm:"default NULL FLOAT"`
	Tauxinfection float64   `json:"TauxInfection" xorm:"default NULL FLOAT"`
	DateString    string    `json:"Date" xorm:"-"`
}

type calc interface {
	GetMaxDate() time.Time
}
